﻿WhoLibFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0")

WhoLibFu.hasIcon = true
WhoLibFu.cannotDetachTooltip = true
WhoLibFu.canHideText = true
WhoLibFu.defaultPosition = "CENTER"

local Tablet = AceLibrary("Tablet-2.0")

local WhoLib = LibStub and LibStub:GetLibrary('LibWho-2.0', true)

local options = {
	type = "group",
	disabled = function() return not WhoLib end,
	args = {
		debug = {
			type = "toggle",
			name = 'WhoLib Debugging',
			desc = 'means: ton\'s on chat log lines',
			get = function()
				return WhoLib:GetWhoLibDebug()
			end,
			set = function(v)
				WhoLib:SetWhoLibDebug(v)
			end,
			order = 1,
		},
		flush = {
			type = "execute",
			name = 'Flush Active Queues',
			desc = 'Clear all pending who operations.',
			func = function()
				WhoLib:Reset()
				WhoLibFu:Update()
			end,
			order = 1,
		},
		snake = {
			type = "toggle",
			name = 'Snake icon',
			desc = 'Yes, the german word "Schlange" means snake and queue...',
			get = function()
				return true
			end,
			set = function()
			end,
			order = 2,
		},
	},
}
WhoLibFu.OnMenuRequest = options

function WhoLibFu:OnEnable()
	WhoLib.RegisterCallback(self, 'WHOLIB_QUERY_ADDED', 'OnWhoEvent')
	WhoLib.RegisterCallback(self, 'WHOLIB_QUERY_RESULT', 'OnWhoEvent')
end

function WhoLibFu:OnWhoEvent(event, ...)
    --print(event, ...)

    WhoLib:UpdateWeights()
    self:Update()
end

function WhoLibFu:OnTextUpdate()
	if WhoLib then
		local text = #WhoLib.Queue[1] .. '/' .. #WhoLib.Queue[2] .. '/' .. #WhoLib.Queue[3] .. (WhoLib.WhoInProgress and " ? " or " - ") .. WhoLib:GetQueryInterval() .. "s"
		self:SetText(text)
	end
end

local tmp = {}


local function getTimeInMinutes(since)
	if since == nil then return "0" end

	local minutes = (time()-since) / 60

	if minutes < 1 then return "< 1" end

	return tostring(math.floor(minutes))
end

function WhoLibFu:OnTooltipUpdate()
	if WhoLib then
	    local cat = Tablet:AddCategory(
	        'columns', 1,
	        'child_textR', 1,
	        'child_textG', 1,
	        'child_textB', 1
	    )
	    
		cat:AddLine(
			'text', '|cff00ff00WhoLib Queues:|r'
		)
	
	    for num,name in ipairs({
	    	' USER',
	    	' QUIET',
	    	' SCANNING',
	    }) do
			cat:AddLine(
				'text', '|cffffff00'..name..'|r ('..(math.floor(WhoLib.queue_bounds[num]*100))..'% chance)'
			)
			if(#WhoLib.Queue[num] == 0)then
					cat:AddLine(
						'text', '|cff888888empty|r'
					)
			else
				for _,v in ipairs(WhoLib.Queue[num]) do
					local text
					if(v.info)then
						text = 'UserInfo: ' .. v.info
					elseif(v.gui)then
						text = 'WhoFrame: ' .. v.query
					elseif(num == 1)then
						text = 'Console: ' .. v.query
					else
						text = 'Who: ' .. v.query
					end
					cat:AddLine(
						'text', text
					)
				end
			end
		end




        wipe(tmp)
        for k,v in pairs(WhoLib.Cache) do tmp[#tmp+1] = k end

		if(#tmp == 0)then
    		cat:AddLine(
    			'text', '|cff00ff00WhoLib Cache:|r'
    		)
			cat:AddLine(
				'text', '|cff888888empty|r'
			)
		else
    		cat:AddLine(
    			'text', ('|cff00ff00WhoLib Cache: %s entries|r'):format(tostring(#tmp))
    		)            
        	table.sort(tmp, function(a, b)
                a, b = WhoLib.Cache[a], WhoLib.Cache[b]
                if a == nil or b == nil then
                    return b ~= nil 
                end              
            
                if a.inqueue ~= b.inqueue then
                    return a.inqueue
                end


                return (a and a.last or 0) > (b and b.last or 0)
        	end )
            local data
            for i, name in ipairs(tmp) do
                data = WhoLib.Cache[name]
                tmp[i]=nil
                if data.inqueue then
    				cat:AddLine(
    					'text', ('%s : |cff8888FFquerying... (%s min)|r'):format(name, getTimeInMinutes(data.last))
    				)    
                else
                    if data.valid then
						if data.data.Online then
	        				cat:AddLine(
	        					'text', ('%s : |cff88FF88%s - %s  (%s min)|r'):format(name, tostring(data.data.Level), tostring(data.data.Class), getTimeInMinutes(data.last))
	        				)
						else
							local status = data.Online == nil and "not found" or "offline"
	        				cat:AddLine(
	        					'text', ('%s : |cff88FF88%s  (%s min)|r'):format(name, status, getTimeInMinutes(data.last))
	        				)
						end
                    else
        				cat:AddLine(
        					'text', ('%s : |cff888888no info|r'):format(name)
        				)
                    end
                end
            end
        end
	end
end

function WhoLibFu:OnClick(button)
end
